//
//  ZKFlutterBridgePlugin.h
//  Runner
//
//  Created by dgg_hzk on 2020/1/2.
//  Copyright © 2020 The Chromium Authors. All rights reserved.
//

#import <Flutter/Flutter.h>

@protocol ZKFlutterBridgeDelegate <NSObject>

@optional
- (void)flutterBridgeCallWithMethod:(NSString *)methodName call:(FlutterMethodCall *)call resultBlock:(FlutterResult)resultBlock;

@end

@interface ZKFlutterBridgePlugin : NSObject<FlutterPlugin>
- (instancetype)init NS_UNAVAILABLE;
+ (instancetype)new NS_UNAVAILABLE;
- (id)copy NS_UNAVAILABLE;
- (id)mutableCopy NS_UNAVAILABLE;

/**
 * 交互代理
 * 1、方式1：设置代理之后，可通过代理来扩展交互方法，也可通过方式2来实现，任选其一
 * 2、方式2：设置代理之后，可直接在代理类中扩展Flutter交互方法，满足methodName:resultBlock:方法选择器规则
 * 3、以上两种方式都有设置时，只会通过一种方式回调，且方式2优先于方式1
 */
@property(nonatomic, weak) id<ZKFlutterBridgeDelegate> delegate;

+ (instancetype)sharedInstance;

/**
 * 执行Flutter方法，没有回调
 */
- (void)invokeFlutterMethod:(NSString *)methodName params:(id)params;

/**
* 执行Flutter方法，有回调
*/
- (void)invokeFlutterMethod:(NSString *)methodName params:(id)params resultBlock:(FlutterResult)resultBlock;

@end
