//
//  ZKFlutterBridgePlugin.m
//  Runner
//
//  Created by dgg_hzk on 2020/1/2.
//  Copyright © 2020 The Chromium Authors. All rights reserved.
//

#import "ZKFlutterBridgePlugin.h"
#include <objc/message.h>

static NSString * const ZK_CHANNEL_NAME = @"zk.flutter.bridge/channel";

@interface ZKFlutterBridgePlugin()

@property (nonatomic, strong) FlutterMethodChannel *methodChannel;

@end

@implementation ZKFlutterBridgePlugin

+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar> *)registrar {
    FlutterMethodChannel *channel = [FlutterMethodChannel methodChannelWithName:ZK_CHANNEL_NAME binaryMessenger:[registrar messenger]];
    ZKFlutterBridgePlugin *plugin = [ZKFlutterBridgePlugin sharedInstance];
    plugin.methodChannel = channel;
    [registrar addMethodCallDelegate:plugin channel:channel];
}

- (id)copyWithZone:(nullable NSZone *)zone {
    return [ZKFlutterBridgePlugin sharedInstance];
}

+ (instancetype)allocWithZone:(struct _NSZone *)zone {
    return [ZKFlutterBridgePlugin sharedInstance];
}

- (id)mutableCopyWithZone:(nullable NSZone *)zone {
    return [ZKFlutterBridgePlugin sharedInstance];
}

+ (instancetype)sharedInstance{
    static ZKFlutterBridgePlugin *_sharedSingleton = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        _sharedSingleton = [[super allocWithZone:NULL] init];
    });
    return _sharedSingleton;
}

- (void)setMethodChannel:(FlutterMethodChannel *)methodChannel{
    _methodChannel = methodChannel;
}

#pragma mark - 通道调用方法处理
- (void)handleMethodCall:(FlutterMethodCall *)call result:(FlutterResult)result{
    NSString *callMethodName = call.method;
    SEL selector = NSSelectorFromString([NSString stringWithFormat:@"%@:resultBlock:",callMethodName]);
    if([self.delegate respondsToSelector:selector]){
        //方式2分发
        void (*nativeFuncation)(id, SEL, id, id) = (void (*)(id, SEL, id, id))objc_msgSend;
        nativeFuncation(self.delegate, selector, call, result);
    }else if ([self.delegate respondsToSelector:@selector(flutterBridgeCallWithMethod:call:resultBlock:)]){
        //方式1分发
        [self.delegate flutterBridgeCallWithMethod:callMethodName call:call resultBlock:result];
    }else{
        if([self respondsToSelector:selector]){
            IMP imp = [self methodForSelector:selector];
            void (*nativeFuncation)(id, SEL, id, id) = (void *)imp;
            nativeFuncation(self, selector, call, result);
        }else{
            result(@{@"code":@"1",@"msg":@"未找到方法",@"data":@""});
        }
    }
}

#pragma mark - 原生调用Flutter方法
- (void)invokeFlutterMethod:(NSString *)methodName params:(id)params{
    if(!self.methodChannel){return;}
    [self.methodChannel invokeMethod:methodName arguments:params result:^(id  _Nullable result) {
        NSLog(@"调用Flutter方法，执行结果 =>：%@", result);
    }];
}

- (void)invokeFlutterMethod:(NSString *)methodName params:(id)params resultBlock:(FlutterResult)resultBlock{
    if(!self.methodChannel){return;}
    [self.methodChannel invokeMethod:methodName arguments:params result:^(id  _Nullable result) {
        NSLog(@"调用Flutter方法，执行结果 =>：%@", result);
        if(resultBlock){
            resultBlock(result);
        }
    }];
}

#pragma mark - 输出一条日志
- (void)printLog:(FlutterMethodCall *)call resultBlock:(FlutterResult)resultBlock{
    NSString *msg = [NSString stringWithFormat:@"%@",call.arguments];
    NSLog(@"flutter log =>: %@", msg);
    resultBlock(@{@"code":@"0",@"msg":@"执行成功"});
}

@end
