/*
 * 
 * 原生与Flutter交互通道单例类
 * 该类统一管理与原生交互操作
 * 
 *  */

import 'package:flutter/services.dart';

//定义回调Block
typedef nativeResponse = void Function(Future<dynamic> result);

const flutterChannelName = 'zk.flutter.bridge/channel';

class FlutterBridge {
  // 工厂模式
  factory FlutterBridge() => _getInstance();
  static FlutterBridge get instance => _getInstance();
  static FlutterBridge _instance;
  static MethodChannel _methodChannel;
  static MethodChannel get methodChannel => getChannel();

  FlutterBridge._internal() {
    //获取通道
    _methodChannel = const MethodChannel(flutterChannelName);
    //设置默认响应原生调用Flutter方法处理，外部可根据需求自定义扩展Flutter方法以供原生调用，通过methodChannel()获取通道
    _methodChannel.setMethodCallHandler((MethodCall call){
      return handleNativeRequest(call);
    });
  }

  static FlutterBridge _getInstance() {
    if (_instance == null) {
      _instance = new FlutterBridge._internal();
    }
    return _instance;
  }

  static MethodChannel getChannel(){
    return _methodChannel;
  }

  Future<dynamic> callNativeWithParams(String method, Object params) async {
    return await _methodChannel.invokeMethod(method, params);
  }

  handleNativeRequest(MethodCall call) async{
    if(call.method == 'printLog'){
      return 'Native log =>：${call.arguments}';
    }else{
      return '未找到对应方法';
    }
  }
}
